//
//  CustomPuzzle.m
//  Demine
//
//  Created by Michael Heyeck on 4/25/10.
//  Copyright 2010 Fair Oaks Labs, Inc.. All rights reserved.
//

#import "UIButton+Glossy.h"
#import "CustomPuzzle.h"


const static NSInteger minEdge = 2;
const static NSInteger maxEdge = 500;


@interface CustomPuzzle ()

- (void)dismiss;

@end


@implementation CustomPuzzle

@synthesize width;
@synthesize height;
@synthesize mines;
@synthesize makeXX;


// The parent's designated initializer. Overriden to call the new DI.
- (id)initWithNibName:(NSString*)nibNameOrNil bundle:(NSBundle*)nibBundleOrNil
{
	return [self initWithNibName:nibNameOrNil bundle:nibBundleOrNil target:nil];
}


// The new designated initializer.
- (id)initWithNibName:(NSString*)nibNameOrNil bundle:(NSBundle*)nibBundleOrNil target:(RootViewController*)aTarget;
{
    if (self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil])
	{
		if (aTarget)
		{
			target = aTarget;
		}
		else
		{
			[self release];
			return nil;
		}
    }
    return self;
}


/*
// Implement loadView to create a view hierarchy programmatically, without using a nib.
- (void)loadView {
}
*/


// Implement viewDidLoad to do additional setup after loading the view, typically from a nib.
- (void)viewDidLoad
{
    [super viewDidLoad];

	// Weird hack - I don't want to talk about it
	UIImage* image = [[UIImage imageNamed:@"bg1.png"] stretchableImageWithLeftCapWidth:0 topCapHeight:2];
	UIImageView* background = [[[UIImageView alloc] initWithImage:image] autorelease];
	background.frame = CGRectMake(0, 0, 320, 416);
	[self.view insertSubview:background atIndex:0];

	// Buttons
	if (![makeXX backgroundImageForState:UIControlStateNormal])
	{
		[makeXX setBackgroundToGlossyRectOfColor:[UIColor purpleColor] withBorder:YES forState:UIControlStateNormal];
	}

	// Titles
	self.title = @"Custom Puzzle";

	// KB
	[width becomeFirstResponder];
}


/*
// Override to allow orientations other than the default portrait orientation.
- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation {
    // Return YES for supported orientations
    return (interfaceOrientation == UIInterfaceOrientationPortrait);
}
*/

- (void)didReceiveMemoryWarning {
	// Releases the view if it doesn't have a superview.
    [super didReceiveMemoryWarning];
	
	// Release any cached data, images, etc that aren't in use.
}

- (void)viewDidUnload {
	// Release any retained subviews of the main view.
	// e.g. self.myOutlet = nil;
}


- (IBAction)makePuzzle:(id)sender
{
	// Pull parameters
	NSInteger w = [width.text integerValue];
	NSInteger h = [height.text integerValue];
	NSInteger m = [mines.text integerValue];
	
	// Validate
	if ((w < minEdge) || (w > maxEdge))
	{
		[width becomeFirstResponder];
		
		NSString* msg = [NSString stringWithFormat:@"You must specify a width between %d and %d",minEdge,maxEdge];
		UIAlertView* av = [[UIAlertView alloc] initWithTitle:nil
													 message:msg
													delegate:nil
										   cancelButtonTitle:@"Ok"
										   otherButtonTitles:nil];
		[av show];
		[av release];
		return;
	}
	if ((h < minEdge) || (h > maxEdge))
	{
		[height becomeFirstResponder];
		
		NSString* msg = [NSString stringWithFormat:@"You must specify a height between %d and %d",minEdge,maxEdge];
		UIAlertView* av = [[UIAlertView alloc] initWithTitle:nil
													 message:msg
													delegate:nil
										   cancelButtonTitle:@"Ok"
										   otherButtonTitles:nil];
		[av show];
		[av release];
		return;
	}
	if ((m < 1) || (m > (w-1)*(h-1)))
	{
		[mines becomeFirstResponder];
		
		NSString* msg = [NSString stringWithFormat:@"You must specify a number of mines between 1 and %d",(w-1)*(h-1)];
		UIAlertView* av = [[UIAlertView alloc] initWithTitle:nil
													 message:msg
													delegate:nil
										   cancelButtonTitle:@"Ok"
										   otherButtonTitles:nil];
		[av show];
		[av release];
		return;
	}
	
	// Build and generate puzzle
	if ([target addPuzzleWithWidth:w height:h mines:m]) [self dismiss];
}


- (void)dealloc
{
	[width release];
	[height release];
	[mines release];
	[makeXX release];
    [super dealloc];
}

#pragma mark Extension methods

- (void)dismiss
{
	[self.navigationController.parentViewController dismissModalViewControllerAnimated:YES];
}

@end
