//
//  DemineAppDelegate.m
//  Demine
//
//  Created by Michael Heyeck on 4/9/10.
//  Copyright Fair Oaks Labs, Inc. 2010. All rights reserved.
//

#import "DemineAppDelegate.h"
#import "RootViewController.h"


@interface DemineAppDelegate ()

- (BOOL)provideContent:(NSString*)productId;

@end


@implementation DemineAppDelegate

@synthesize window;
@synthesize navigationController;


#pragma mark -
#pragma mark Application lifecycle

- (void)applicationDidFinishLaunching:(UIApplication*)application
{   
    // Override point for customization after app launch
	[[SKPaymentQueue defaultQueue] addTransactionObserver:self];

	RootViewController *rootViewController = (RootViewController *)[navigationController topViewController];
	rootViewController.managedObjectContext = self.managedObjectContext;
	
	[window addSubview:[navigationController view]];
    [window makeKeyAndVisible];
}

/**
 applicationWillTerminate: saves changes in the application's managed object context before the application terminates.
 */
- (void)applicationWillTerminate:(UIApplication *)application {
	
    NSError *error = nil;
    if (managedObjectContext != nil) {
        if ([managedObjectContext hasChanges] && ![managedObjectContext save:&error]) {
			/*
			 Replace this implementation with code to handle the error appropriately.
			 */
			NSLog(@"Unresolved error %@, %@", error, [error userInfo]);
        } 
    }
}


#pragma mark -
#pragma mark Core Data stack

/**
 Returns the managed object context for the application.
 If the context doesn't already exist, it is created and bound to the persistent store coordinator for the application.
 */
- (NSManagedObjectContext *) managedObjectContext {
	
    if (managedObjectContext != nil) {
        return managedObjectContext;
    }
	
    NSPersistentStoreCoordinator *coordinator = [self persistentStoreCoordinator];
    if (coordinator != nil) {
        managedObjectContext = [[NSManagedObjectContext alloc] init];
        [managedObjectContext setPersistentStoreCoordinator: coordinator];
    }
    return managedObjectContext;
}


/**
 Returns the managed object model for the application.
 If the model doesn't already exist, it is created by merging all of the models found in the application bundle.
 */
- (NSManagedObjectModel *)managedObjectModel {
	
    if (managedObjectModel != nil) {
        return managedObjectModel;
    }
    managedObjectModel = [[NSManagedObjectModel mergedModelFromBundles:nil] retain];    
    return managedObjectModel;
}


/**
 Returns the persistent store coordinator for the application.
 If the coordinator doesn't already exist, it is created and the application's store added to it.
 */
- (NSPersistentStoreCoordinator *)persistentStoreCoordinator {
	
    if (persistentStoreCoordinator != nil) {
        return persistentStoreCoordinator;
    }
	
    NSURL *storeUrl = [NSURL fileURLWithPath: [[self applicationDocumentsDirectory] stringByAppendingPathComponent: @"Demine.sqlite"]];
	NSDictionary* options = [NSDictionary dictionaryWithObjectsAndKeys:[NSNumber numberWithBool:YES],NSMigratePersistentStoresAutomaticallyOption,
																	   [NSNumber numberWithBool:YES],NSInferMappingModelAutomaticallyOption,
																	   nil];
	
	NSError *error = nil;
    persistentStoreCoordinator = [[NSPersistentStoreCoordinator alloc] initWithManagedObjectModel:[self managedObjectModel]];
    if (![persistentStoreCoordinator addPersistentStoreWithType:NSSQLiteStoreType configuration:nil URL:storeUrl options:options error:&error])
	{
		/*
		 Typical reasons for an error here include:
		 * The persistent store is not accessible
		 * The schema for the persistent store is incompatible with current managed object model
		 Check the error message to determine what the actual problem was.
		 */
		NSLog(@"Unresolved error %@, %@", error, [error userInfo]);
		
		UIAlertView* av = [[UIAlertView alloc] initWithTitle:nil
													 message:@"An unrecoverable error occurred.  Please press the Home button to exit."
													delegate:nil
										   cancelButtonTitle:nil
										   otherButtonTitles:nil];
		[av show];
		[av release];
    }    
	
    return persistentStoreCoordinator;
}


#pragma mark -
#pragma mark Application's Documents directory

/**
 Returns the path to the application's Documents directory.
 */
- (NSString *)applicationDocumentsDirectory {
	return [NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES) lastObject];
}


#pragma mark -
#pragma mark Memory management

- (void)dealloc
{
	[[SKPaymentQueue defaultQueue] removeTransactionObserver:self];
	
    [managedObjectContext release];
    [managedObjectModel release];
    [persistentStoreCoordinator release];
    
	[navigationController release];
	[window release];
	[super dealloc];
}


#pragma mark -
#pragma mark Store kit

- (void)paymentQueue:(SKPaymentQueue*)queue updatedTransactions:(NSArray*)transactions
{
	for (SKPaymentTransaction* transaction in transactions)
	{
		switch (transaction.transactionState)
		{
			case SKPaymentTransactionStatePurchased:
				if ([self provideContent:transaction.payment.productIdentifier])
				{
					NSString* msg = @"In-app purchase completed.  You might want to back up your device to preserve this purchase.";
					UIAlertView* av = [[UIAlertView alloc] initWithTitle:nil
																 message:msg
																delegate:nil
													   cancelButtonTitle:@"Ok"
													   otherButtonTitles:nil];
					[av show];
					[av release];
				}
				[queue finishTransaction:transaction];
				break;
			case SKPaymentTransactionStateFailed:
				if (transaction.error.code != SKErrorPaymentCancelled)
				{
					NSString* msg = @"We're sorry, but there was a problem with an in-app purchase.";
					UIAlertView* av = [[UIAlertView alloc] initWithTitle:nil
																 message:msg
																delegate:nil
													   cancelButtonTitle:@"Ok"
													   otherButtonTitles:nil];
					[av show];
					[av release];
				}
				[queue finishTransaction:transaction];
				break;
			case SKPaymentTransactionStateRestored:
				[self provideContent:transaction.originalTransaction.payment.productIdentifier];
				[queue finishTransaction:transaction];
				break;
			default:
				break;
		}
	}
}


- (void)paymentQueueRestoreCompletedTransactionsFinished:(SKPaymentQueue*)queue
{
	NSString* msg = @"All purchases have been restored.  You might want to back up your device to preserve them.";
	UIAlertView* av = [[UIAlertView alloc] initWithTitle:nil
												 message:msg
												delegate:nil
									   cancelButtonTitle:@"Ok"
									   otherButtonTitles:nil];
	[av show];
	[av release];
}


- (void)paymentQueue:(SKPaymentQueue*)queue restoreCompletedTransactionsFailedWithError:(NSError*)error
{
	NSString* msg = @"An error occurred while restoring purchases.";
	UIAlertView* av = [[UIAlertView alloc] initWithTitle:nil
												 message:msg
												delegate:nil
									   cancelButtonTitle:@"Ok"
									   otherButtonTitles:nil];
	[av show];
	[av release];
}


#pragma mark -
#pragma mark Extensions

- (BOOL)provideContent:(NSString*)productId
{
	if ([productId isEqualToString:@"com.fairoakslabs.Demine.unlock0"])
	{
		NSUserDefaults* defaults = [NSUserDefaults standardUserDefaults];
		[defaults setInteger:1 forKey:@"enableAllPuzzles"];
		[defaults synchronize];
		return TRUE;
	}
	return FALSE;
}

@end

