//
//  HighScores.m
//  Demine
//
//  Created by Michael Heyeck on 4/25/10.
//  Copyright 2010 Fair Oaks Labs, Inc.. All rights reserved.
//

#import "HighScore.h"
#import "HighScores.h"


@interface HighScores ()

@property (nonatomic, retain, readonly) NSFetchedResultsController* fetchedResultsController;
@property (nonatomic, retain, readonly) NSDateFormatter* timerFormatter;

- (NSString*)formatTimer:(NSTimeInterval)timeInterval;
- (void)dismiss;

@end


@implementation HighScores

@synthesize managedObjectContext;

- (NSFetchedResultsController*)fetchedResultsController
{
    if (!fetchedResultsController)
	{
		// Create the fetch request that will drive the controller
		NSFetchRequest* fetchRequest = [[[NSFetchRequest alloc] init] autorelease];

		// Edit the entity name
		fetchRequest.entity = [NSEntityDescription entityForName:@"HighScore" inManagedObjectContext:managedObjectContext];
		
		// Set the batch size
		fetchRequest.fetchBatchSize = 20;
		
		// Edit the sort key as appropriate.
		NSSortDescriptor* sortDescriptor0 = [[[NSSortDescriptor alloc] initWithKey:@"width" ascending:YES] autorelease];
		NSSortDescriptor* sortDescriptor1 = [[[NSSortDescriptor alloc] initWithKey:@"height" ascending:YES] autorelease];
		NSSortDescriptor* sortDescriptor2 = [[[NSSortDescriptor alloc] initWithKey:@"mines" ascending:YES] autorelease];
		fetchRequest.sortDescriptors = [[[NSArray alloc] initWithObjects:sortDescriptor0,sortDescriptor1,sortDescriptor2,nil] autorelease];
		
		// Create the controller
		fetchedResultsController = [[NSFetchedResultsController alloc] initWithFetchRequest:fetchRequest
																	   managedObjectContext:managedObjectContext
																		 sectionNameKeyPath:nil
																				  cacheName:nil];
    }
	return fetchedResultsController;
}

- (NSDateFormatter*)timerFormatter
{
	if (!timerFormatter)
	{
		timerFormatter = [[NSDateFormatter alloc] init];
		timerFormatter.dateFormat = @"H:mm:ss";
		timerFormatter.timeZone = [NSTimeZone timeZoneForSecondsFromGMT:0];
	}
	return timerFormatter;
}


/*
- (id)initWithStyle:(UITableViewStyle)style {
    // Override initWithStyle: if you create the controller programmatically and want to perform customization that is not appropriate for viewDidLoad.
    if (self = [super initWithStyle:style]) {
    }
    return self;
}
*/


- (void)viewDidLoad
{
    [super viewDidLoad];

	self.navigationItem.leftBarButtonItem = [[[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemDone target:self action:@selector(dismiss)] autorelease];
	
	self.title = @"Best Times";
	
	NSError* err;
	[self.fetchedResultsController performFetch:&err];
}


/*
- (void)viewWillAppear:(BOOL)animated {
    [super viewWillAppear:animated];
}
*/
/*
- (void)viewDidAppear:(BOOL)animated {
    [super viewDidAppear:animated];
}
*/
/*
- (void)viewWillDisappear:(BOOL)animated {
	[super viewWillDisappear:animated];
}
*/
/*
- (void)viewDidDisappear:(BOOL)animated {
	[super viewDidDisappear:animated];
}
*/

/*
// Override to allow orientations other than the default portrait orientation.
- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation {
    // Return YES for supported orientations
    return (interfaceOrientation == UIInterfaceOrientationPortrait);
}
*/

- (void)didReceiveMemoryWarning {
	// Releases the view if it doesn't have a superview.
    [super didReceiveMemoryWarning];
	
	// Release any cached data, images, etc that aren't in use.
}

- (void)viewDidUnload {
	// Release any retained subviews of the main view.
	// e.g. self.myOutlet = nil;
}


- (void)dealloc
{
	[managedObjectContext release];
	[fetchedResultsController release];
	[timerFormatter release];
    [super dealloc];
}

#pragma mark Table view data source methods

- (NSInteger)numberOfSectionsInTableView:(UITableView*)tableView
{
    return 1;
}


// Customize the number of rows in the table view.
- (NSInteger)tableView:(UITableView*)tableView numberOfRowsInSection:(NSInteger)section
{
	NSArray* sections = [fetchedResultsController sections];
	if ([sections count])
	{
		return [[sections objectAtIndex:section] numberOfObjects];
	}
	else
	{
		return 0;
	}
}


// Customize the appearance of table view cells.
- (UITableViewCell*)tableView:(UITableView*)tableView cellForRowAtIndexPath:(NSIndexPath*)indexPath
{
    static NSString* CellIdentifier = @"Cell";
    
    UITableViewCell* cell = [tableView dequeueReusableCellWithIdentifier:CellIdentifier];
    if (cell == nil)
	{
        cell = [[[UITableViewCell alloc] initWithStyle:UITableViewCellStyleValue1 reuseIdentifier:CellIdentifier] autorelease];
    }
    
	HighScore* hs = [fetchedResultsController objectAtIndexPath:indexPath];
	NSString* size = [NSString stringWithFormat:@"%d x %d (%d)",[hs.width integerValue],[hs.height integerValue],[hs.mines integerValue]];
	NSString* time = [self formatTimer:[hs.best doubleValue]];
	
	cell.textLabel.text = size;
	cell.detailTextLabel.text = time;
	
    return cell;
}


/*
// Override to support conditional editing of the table view.
- (BOOL)tableView:(UITableView *)tableView canEditRowAtIndexPath:(NSIndexPath *)indexPath {
    // Return NO if you do not want the specified item to be editable.
    return YES;
}
*/


/*
// Override to support editing the table view.
- (void)tableView:(UITableView *)tableView commitEditingStyle:(UITableViewCellEditingStyle)editingStyle forRowAtIndexPath:(NSIndexPath *)indexPath {
    
    if (editingStyle == UITableViewCellEditingStyleDelete) {
        // Delete the row from the data source
        [tableView deleteRowsAtIndexPaths:[NSArray arrayWithObject:indexPath] withRowAnimation:YES];
    }   
    else if (editingStyle == UITableViewCellEditingStyleInsert) {
        // Create a new instance of the appropriate class, insert it into the array, and add a new row to the table view
    }   
}
*/


/*
// Override to support rearranging the table view.
- (void)tableView:(UITableView *)tableView moveRowAtIndexPath:(NSIndexPath *)fromIndexPath toIndexPath:(NSIndexPath *)toIndexPath {
}
*/


/*
// Override to support conditional rearranging of the table view.
- (BOOL)tableView:(UITableView *)tableView canMoveRowAtIndexPath:(NSIndexPath *)indexPath {
    // Return NO if you do not want the item to be re-orderable.
    return YES;
}
*/

#pragma mark Table view delegate methods

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
    // Navigation logic may go here. Create and push another view controller.
	// AnotherViewController *anotherViewController = [[AnotherViewController alloc] initWithNibName:@"AnotherView" bundle:nil];
	// [self.navigationController pushViewController:anotherViewController];
	// [anotherViewController release];
}


- (NSIndexPath*)tableView:(UITableView*)tableView willSelectRowAtIndexPath:(NSIndexPath*)indexPath
{
	// Prohibit row selection
	return nil;
}

#pragma mark Extension methods

- (NSString*)formatTimer:(NSTimeInterval)timeInterval
{
	return [self.timerFormatter stringFromDate:[NSDate dateWithTimeIntervalSinceReferenceDate:timeInterval]];
}


- (void)dismiss
{
	[self.navigationController.parentViewController dismissModalViewControllerAnimated:YES];
}

@end

