//
//  Item.m
//  Demine
//
//  Created by Michael Heyeck on 5/3/10.
//  Copyright 2010 Fair Oaks Labs, Inc.. All rights reserved.
//

#import "UIButton+Glossy.h"
#import "Item.h"


@implementation Item

@synthesize desc;
@synthesize button;
@synthesize loadingView;


// The parent's designated initializer. Overriden to call the new DI.
- (id)initWithNibName:(NSString*)nibNameOrNil bundle:(NSBundle*)nibBundleOrNil
{
	return [self initWithNibName:nibNameOrNil bundle:nibBundleOrNil product:nil];
}

 
// The new designated initializer.
- (id)initWithNibName:(NSString*)nibNameOrNil bundle:(NSBundle*)nibBundleOrNil product:(SKProduct*)aProduct
{
	if (self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil])
	{
		if (aProduct)
		{
			product = [aProduct retain];
		}
		else
		{
			[self release];
			return nil;
		}
	}
	return self;
}


/*
// Implement loadView to create a view hierarchy programmatically, without using a nib.
- (void)loadView {
}
*/


// Implement viewDidLoad to do additional setup after loading the view, typically from a nib.
- (void)viewDidLoad
{
    [super viewDidLoad];

	// Weird hack - I don't want to talk about it
	UIImage* image = [[UIImage imageNamed:@"bg2.png"] stretchableImageWithLeftCapWidth:0 topCapHeight:198];
	UIImageView* background = [[[UIImageView alloc] initWithImage:image] autorelease];
	background.frame = CGRectMake(0, 0, 320, 416);
	[self.view insertSubview:background atIndex:0];

	// Helper for price formatting
	NSNumberFormatter* pf = [[[NSNumberFormatter alloc] init] autorelease];
	pf.formatterBehavior = NSNumberFormatterBehavior10_4;
	pf.numberStyle = NSNumberFormatterCurrencyStyle;
	pf.locale = product.priceLocale;

	// Title and description
	self.title = [NSString stringWithFormat:@"%@ %@",product.localizedTitle,[pf stringFromNumber:product.price]];
	desc.text = product.localizedDescription;

	if (![button backgroundImageForState:UIControlStateNormal])
	{
		[button setBackgroundToGlossyRectOfColor:[UIColor colorWithRed:0.05 green:0.6 blue:0.05 alpha:1] withBorder:YES forState:UIControlStateNormal];
	}
}


/*
// Override to allow orientations other than the default portrait orientation.
- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation {
    // Return YES for supported orientations
    return (interfaceOrientation == UIInterfaceOrientationPortrait);
}
*/

- (void)didReceiveMemoryWarning {
	// Releases the view if it doesn't have a superview.
    [super didReceiveMemoryWarning];
	
	// Release any cached data, images, etc that aren't in use.
}

- (void)viewDidUnload {
	// Release any retained subviews of the main view.
	// e.g. self.myOutlet = nil;
}


- (IBAction)purchase:(id)sender
{
	if (![SKPaymentQueue canMakePayments])
	{
		NSString* msg = @"You have disabled in-app purchases; please re-enable them to complete this purchase.";
		UIAlertView* av = [[UIAlertView alloc] initWithTitle:nil
													 message:msg
													delegate:nil
										   cancelButtonTitle:@"Ok"
										   otherButtonTitles:nil];
		[av show];
		[av release];
		return;
	}
	
	self.loadingView.hidden = NO;
	[[SKPaymentQueue defaultQueue] addTransactionObserver:self];
	[[SKPaymentQueue defaultQueue] addPayment:[SKPayment paymentWithProduct:product]];
}


- (void)dealloc
{
	[[SKPaymentQueue defaultQueue] removeTransactionObserver:self];

	[product release];
	[desc release];
	[button release];
	[loadingView release];
    [super dealloc];
}

#pragma mark Store kit observer methods

- (void)paymentQueue:(SKPaymentQueue*)queue updatedTransactions:(NSArray*)transactions
{
	for (SKPaymentTransaction* transaction in transactions)
	{
		switch (transaction.transactionState)
		{
			case SKPaymentTransactionStatePurchased:
			case SKPaymentTransactionStateFailed:
				if ([transaction.payment.productIdentifier isEqualToString:product.productIdentifier])
				{
					// Sort of specialized, here ...
					[self.navigationController.parentViewController dismissModalViewControllerAnimated:YES];
				}
				break;
			default:
				break;
		}
	}
}

@end
